#ifndef darksidecore_Data_DS50V1495FRAGMENT_hh
#define darksidecore_Data_DS50V1495FRAGMENT_hh

#include "artdaq-core/Data/Fragment.hh"
#include <ostream>
#include <sys/time.h>

// Fragment overlay class for DS50 data.
namespace ds50 {
  class V1495Driver;
  class V1495Fragment {
    public:
//#pragma pack(push,2)
      struct metadata {
	timeval trigger_time;
	uint16_t serial;
	uint8_t firmware_version;
	int8_t threshold;
	uint32_t inhibit_20ns;
	uint16_t g2_threshold;
	uint8_t g2_gate_20ns, g2_prescale;
	uint16_t g2_threshold_high;
	uint8_t unused[2];
      };
//#pragma pack(pop) 
      static_assert (sizeof (metadata) == 32, "metadata size changed");

      enum trigger_types {
        tpc_high = 1,
	laser = 2,
	external = 3,
	pulser = 4,
	random = 5,
        tpc_low = 6,
	tpc = 7,
	veto_ls = 8,
	veto_cw = 9,
      };
      typedef uint16_t run_number_t;
      typedef uint16_t trigger_id_t;
      typedef uint32_t register_t;
      typedef uint64_t trigger_pattern_t;
      typedef uint32_t count_t;

      explicit V1495Fragment(artdaq::Fragment const & f);

      static size_t pod_size () { return sizeof (POD); }

      size_t event_size () const { return pod_ ()->length * sizeof (uint32_t); }

      run_number_t run_number () const { return pod_ ()->run_number; }

      trigger_types trigger_type () const { return trigger_types((pod_ ()->trigger_id >> 12) & 0xf); }
      trigger_id_t trigger_ordinal () const { return pod_ ()->trigger_id & 0xfff; }

      count_t trigger_counter () const { return pod_ ()->trigger_counter; }

      register_t status_register () const { return pod_ ()->status_register; }
      register_t trigger_mask () const { return pod_ ()->trigger_mask; }
      register_t majority_setup () const { return pod_ ()->majority_setup; }

      count_t tpc_he_events_prescale () const { return pod_ ()->tpc_he_events_prescale; }
      count_t tpc_digital_sum () const { return pod_ ()->tpc_digital_sum; }

      int board_id () const { return pod_ ()->board_id; }
      count_t gps_coarse_s () const { return pod_ ()->gps_coarse_s; }
      count_t gps_fine_20ns () const { return pod_ ()->gps_fine_20ns; }
      count_t gps_pps_20ns () const { return pod_ ()->gps_pps_20ns; }

      trigger_pattern_t trigger_pattern () const { return (trigger_pattern_t(pod_ ()->tpc_trigger_pattern_high) << 32) | pod_ ()->tpc_trigger_pattern_low; }
      bool channel_in_trigger (int ch) const;
      unsigned int trigger_multiplicity () const;

      count_t total_inhibit_time_us () const { return pod_ ()->total_inhibit_time_us; }
      count_t incremental_inhibit_time_100ns () const { return pod_ ()->incremental_inhibit_time_100ns / ((run_number () >= 5800) ? 1 : 5); }
      count_t live_time_100ns () const { return pod_ ()->live_time_100ns / ((run_number () >= 5800) ? 1 : 5); }

      bool fast_verify () const;

    protected:
      struct POD {
	uint32_t length : 8, fw_type : 8, run_number : 16;
	uint32_t trigger_id : 16, status_register : 16;
	uint32_t trigger_mask : 16, majority_setup : 16;
	uint32_t gps_coarse_s : 24, board_id : 8;
	uint32_t gps_fine_20ns;
	uint32_t gps_pps_20ns;
	uint32_t tpc_trigger_pattern_low;
	uint32_t tpc_trigger_pattern_high : 8, tpc_he_events_prescale : 8, tpc_digital_sum : 16;
	uint32_t trigger_counter;
	uint32_t unused : 8, zero6 : 24;
	uint32_t total_inhibit_time_us;
	uint32_t incremental_inhibit_time_100ns;
	uint32_t live_time_100ns;
      };

      POD const * pod_ () const;
      friend class V1495Driver;
    private:
      artdaq::Fragment const & data_;
  };

  std::ostream & operator << (std::ostream &, V1495Fragment const &);
}
inline ds50::V1495Fragment::V1495Fragment(artdaq::Fragment const & f): data_(f) {}

inline ds50::V1495Fragment::POD const * ds50::V1495Fragment::pod_() const {
  return reinterpret_cast<V1495Fragment::POD const *>(&*data_.dataBegin());
}

inline bool ds50::V1495Fragment::fast_verify () const { 
  return pod_size () == event_size () && ((pod_ ()->trigger_id & 0xfff) == (trigger_counter () & 0xfff));
}

#endif
