#include "art/Framework/Core/EDAnalyzer.h"
#include "art/Framework/Core/ModuleMacros.h"
#include "art/Framework/Principal/Event.h"
#include "art/Framework/Principal/Handle.h"
#include "art/Framework/Principal/Run.h"
#include "art/Framework/Principal/SubRun.h"
#include "art/Persistency/Provenance/BranchType.h"
#include "art/Persistency/Provenance/EventID.h"
#include "darksidecore/Compression/Codes.hh"
#include "darksidecore/Compression/Properties.hh"
#include "darksidecore/Compression/SymTable.hh"
#include "darksidecore/Compression/SymProb.hh"
#include "darksidecore/Data/V172xFragment.hh"
#include "artdaq-core/Data/Fragment.hh"
#include "artdaq-core/Data/Fragments.hh"
#include "cpp0x/memory"
#include "fhiclcpp/ParameterSet.h"
#include "messagefacility/MessageLogger/MessageLogger.h"

#include <cstdlib>
#include <iostream>
#include <list>
#include <ostream>
#include <fstream>
#include <sstream>
#include <string>
#include <utility>

using std::vector;
using namespace std;

namespace {

}

namespace ds50 {

  class DS50ExpProbs : public art::EDAnalyzer
  {
  public:
    explicit DS50ExpProbs(fhicl::ParameterSet const & p);
    virtual ~DS50ExpProbs() { }
    
    virtual void analyze(art::Event const & e);
    virtual void endSubRun(art::SubRun const & sr);
    virtual void endRun(art::Run const & r);

  private:
    std::string const mod_label_;
    std::string const raw_label_;
    std::string const inst_name_;
    size_t bits_;
    DataVec counts_;
  };
  
  DS50ExpProbs::DS50ExpProbs(fhicl::ParameterSet const & p):
    art::EDAnalyzer(p),
    mod_label_(p.get<std::string>("module_label")),
    raw_label_(p.get<std::string>("raw_label","daq")),
    inst_name_(p.get<std::string>("instance")),
    bits_(inst_name_=="V1724" ? 14 : inst_name_=="V1720" ? 12 : 0),
    counts_(50)
  {
  }

  void DS50ExpProbs::analyze(art::Event const & e)
  {
    art::Handle<artdaq::Fragments> handle;
    e.getByLabel(raw_label_, inst_name_, handle);
    size_t const len = handle->size();

    for (size_t i = 0; i < len; ++i) 
      {
	artdaq::Fragment const & frag = (*handle)[i];
	V172xFragment b(frag);
	// start of payload is the DS50 header
	auto adc_start = b.dataBegin();
	auto adc_end   = b.dataEnd();

	unsigned long run_med, one_bit_total, total_bits;
	ADCCountVec vals(adc_start,adc_end);

	ds50::calculateRunLengths(bits_,
				  vals, counts_,
				  run_med, one_bit_total, total_bits);

	cout << "ExpProb:sum"
	     << " " << mod_label_
	     << " " << raw_label_
	     << " " << inst_name_
	     << " " << e.event()
	     << " " << i
	     << " " << run_med
	     << " " << one_bit_total 
	     << " " << total_bits
	     << "\n";

      }
  }

  void DS50ExpProbs::endSubRun(art::SubRun const &) { }

  void DS50ExpProbs::endRun(art::Run const & r)
  {
    for(size_t i=0;i<counts_.size();++i)
      {
	cout << "ExpProb:counts"
	     << " " << mod_label_
	     << " " << r.run()
	     << " " << i
	     << " " << counts_[i]
	     << "\n";
      }
  }

  DEFINE_ART_MODULE(DS50ExpProbs)
}

