#include "art/Framework/Core/EDProducer.h"
#include "art/Framework/Core/ModuleMacros.h"
#include "art/Framework/Principal/Event.h"
#include "art/Framework/Principal/Handle.h"
#include "art/Framework/Principal/Run.h"
#include "art/Framework/Principal/SubRun.h"
#include "art/Persistency/Provenance/BranchType.h"
#include "art/Persistency/Provenance/EventID.h"
#include "darksidecore/Compression/Encoder.hh"
#include "darksidecore/Compression/Codes.hh"
#include "darksidecore/Compression/Properties.hh"
#include "darksidecore/Compression/SymTable.hh"
#include "darksidecore/Data/V172xFragment.hh"
#include "darksidecore/Data/CompressedV172x.hh"
#include "artdaq-core/Data/Fragment.hh"
#include "artdaq-core/Data/Fragments.hh"
#include "cpp0x/memory"
#include "fhiclcpp/ParameterSet.h"
#include "messagefacility/MessageLogger/MessageLogger.h"

#include <cstdlib>
#include <iostream>
#include <list>
#include <ostream>
#include <fstream>
#include <sstream>
#include <string>
#include <utility>

using std::vector;
using namespace std;

namespace {
}

namespace ds50 {

  class Shifter : public art::EDProducer 
  {
  public:
    explicit Shifter(fhicl::ParameterSet const & p);
    virtual ~Shifter() { }
    
    virtual void produce(art::Event & e);
    virtual void endSubRun(art::SubRun & sr);
    virtual void endRun(art::Run & r);

  private:
    std::string const raw_label_;
    std::string const inst_name_;
    int shift_;
    int count_max_;
  };
  
  Shifter::Shifter(fhicl::ParameterSet const & p):
    raw_label_(p.get<std::string>("raw_label","daq")),
    inst_name_(p.get<std::string>("instance_name")),
    shift_(p.get<int>("shift")),
    count_max_(inst_name_=="V1720" ? V1720Properties::count_max() : 
	       inst_name_=="V1724" ? V1724Properties::count_max() :
	       0)
  {
    produces<artdaq::Fragments>(inst_name_);

    if(count_max_ == 0)
      {
	throw std::runtime_error("invalid instance name (V1720/V1724 good)");
      }
  }

  void Shifter::produce(art::Event & e)
  {
    art::Handle<artdaq::Fragments> handle;
    e.getByLabel(raw_label_, inst_name_, handle);
    // copy the entire thing
    std::unique_ptr<artdaq::Fragments> prod(new artdaq::Fragments(*handle));
    size_t const len = prod->size();

    for (size_t i = 0; i < len; ++i) 
      {
	artdaq::Fragment& frag = (*prod)[i];
	// V172xFragment b(frag);
	// start of payload is the DS50 header
	auto adc_start = frag.dataBegin();
	auto adc_end   = frag.dataEnd();

	for(auto pos=adc_start; pos!=adc_end; ++pos)
	  {
	    int mod_val = (int)*pos + shift_;
	    if(mod_val>=0 && mod_val <count_max_)
	      *pos = (adc_type)mod_val;
	  }
      }
    // need to include bias and algorithm name to the product
    e.put(std::move(prod),inst_name_);
  }

  void Shifter::endSubRun(art::SubRun &) { }

  void Shifter::endRun(art::Run &)
  {
  }

  DEFINE_ART_MODULE(Shifter)
}

