#include "art/Framework/Principal/Run.h"
#include "art/Framework/Services/Registry/ActivityRegistry.h"
#include "fhiclcpp/ParameterSet.h"

#include "darksidecore/ArtServices/DBInterface.hh"

ds50::DBInterface::DBInterface (const fhicl::ParameterSet &ps, art::ActivityRegistry & registry): db::interface(ps.get<std::string> ("hostname"), ps.get<std::string> ("port", "5432"), ps.get<std::string> ("database"), ps.get<std::string> ("user"), ps.get<std::string> ("password"), ps.get<std::string> ("write_hostname", ""), ps.get<std::string> ("write_port", "5432"), ps.get<bool> ("verbose", false)) {
  registry.sPreBeginRun.watch(this, &DBInterface::pre_begin_run);
  registry.sPostBeginRun.watch(this, &DBInterface::post_begin_run);
  registry.sPreEndRun.watch(this, &DBInterface::pre_end_run);
  set_current_run (-1);
}

void ds50::DBInterface::pre_begin_run (const art::Run& run) {
  disconnect ();
  set_current_run (run.run ());
}

void ds50::DBInterface::post_begin_run (const art::Run&) {
  disconnect ();
}

void ds50::DBInterface::pre_end_run (const art::RunID&, const art::Timestamp&) {
  disconnect ();
}


DEFINE_ART_SERVICE(ds50::DBInterface)
