#ifndef _DS50DB_INTERFACE
#define _DS50DB_INTERFACE
#include <string>
#include <memory>
#include <pqxx/pqxx>
#include <mutex>

#include "darksidecore/ArtServices/ds50db_result.hh"

namespace ds50 {
  namespace db {
    class interface {
      public:
	interface (const std::string& host, const std::string& port, const std::string& db, const std::string& user, const std::string& pwd, \
	           const std::string& write_host, const std::string& write_port, bool verbose);

	int current_run () const;

	int latest_run (const std::string& table, const std::string& version = "");

	int latest_run (int run, const std::string& table, const std::string& version = "");

	result latest (const std::string& table, const std::string& version = "");

	result latest (int run, const std::string& table, const std::string& version = "");

	result current (const std::string& table, const std::string& version = "");

	result run (int run, const std::string& table, const std::string& version = "");

	result insert (const result& r, const std::string& version);

	template<typename T> std::string quote (const T& t) const;

	result query (const std::string& q, bool autocommit = false);

	typedef pqxx::lazyconnection connection;
	class work {
	  public:
	    work (connection& c);
	    result exec (const std::string& q);
	    void commit ();
	  private:
	    pqxx::work w_;
	    std::lock_guard<std::mutex> lk_;
	    static std::mutex mutex_;
	};

	std::unique_ptr<work> transaction ();

      protected:
	void set_current_run (int run);
	void disconnect ();

      private:
	std::unique_ptr<connection> c_, write_c_;
	bool verbose_;
	int current_run_;
	int timeout_;

	std::unique_ptr<work> transaction (connection& c);
    };
    std::mutex interface::work::mutex_;
  }
}

inline int ds50::db::interface::current_run () const { return current_run_; }

inline int ds50::db::interface::latest_run (const std::string& table, const std::string& version) { return latest_run (current_run_, table, version); }

inline ds50::db::result ds50::db::interface::latest (const std::string& table, const std::string& version) { return latest (current_run_, table, version); }

inline ds50::db::result ds50::db::interface::current (const std::string& table, const std::string& version) { return run (current_run_, table, version); }

template<typename T> std::string ds50::db::interface::quote (const T& t) const { 
  try {
    return c_->quote (t); 
  } catch (const std::runtime_error& e) {
    throw cet::exception("dbi::quote") << e.what ();
  }
}

inline void ds50::db::interface::set_current_run (int run) { current_run_ = run; }

inline std::unique_ptr<ds50::db::interface::work> ds50::db::interface::transaction () { return transaction (*c_); }
#endif
