#include "darksidecore/ArtServices/ds50db_result.hh"
#include <pqxx/pqxx>
#include <boost/tokenizer.hpp>

ds50::db::result::result (const pqxx::result& r): query_(r.query ()) {
  pqxx::tuple::size_type cols = r.columns ();
  for (pqxx::tuple::size_type i = 0; i < cols; i++) cells_[r.column_name (i)];
  if (r.empty ()) return;
  for (const auto &row: r) {
    for (const auto &element: row) {
      std::string column_name(element.name ());
      size_t row_number(row.rownumber ());
      std::string value(element.c_str ());
      if (row_number >= cells_[column_name].size ()) cells_[column_name].resize (row_number + 1);
      cells_[column_name][row_number].first = value;
      if (value.size () && value[0] == '{' && value[value.size () - 1] == '}') {
	std::string text = value.substr(1, value.size() - 2);
	boost::tokenizer<boost::escaped_list_separator<char>> tokens(text);
	for (const auto& t : tokens) cells_[column_name][row_number].second.push_back (t);
      }
    }
  }
} 

const std::string& ds50::db::result::column_name (size_t column) const {
  if (column >= columns ()) throw cet::exception("dbi::set") << "column " << column << " does not exist" << (table_name_.size () ? " in table " : "") << table_name_;
  auto c = cells_.begin ();
  std::advance (c, column);
  return c->first;
}
std::vector<std::string> ds50::db::result::column_names () const { 
  std::vector<std::string> names;
  for (const auto &x: cells_) names.push_back (x.first);
  return names;
}

bool ds50::db::result::check_cell (const std::string& column, size_t row) const {
  auto x = cells_.find (column);
  if (x == cells_.end ()) return false;
  if (row >= x->second.size ()) return false;
  return true;
}

bool ds50::db::result::check_cell (size_t column, size_t row) const {
  if (column >= cells_.size ()) return false;
  auto c = cells_.begin ();
  std::advance (c, column);
  if (row >= c->second.size ()) return false;
  return true;
}

void ds50::db::result::wipe () {
  for (auto &x: cells_) x.second.clear ();
}
